#********************************************************************************
# qrtools postsync hook                                                         *
#                                                                               *
#  Author: Kai Peter, ©2016-                                                    *
# Version: 0.2                                                                  *
# License: ISC                                                                  *
#                                                                               *
# Description: Run some postsync hooks for repository gentoo.                   *
#********************************************************************************
CFGFILE="${PORTAGE_CONFIGROOT%/}/etc/portage/qrtools.conf/postsync.conf"
ME=`basename $0`

DATE=`date +'%b %d %H:%M:%S'`
LOGFILE="/var/log/qrt-postsync.log"
DBG=0

#set > set-postsync.env

# The next variables comes from protage:
REPONAME=${1}		# repository name
SYNC_URI=${2}		# URI (to which the repository was synced)
REPOPATH=${3}		# repository path

[ ${REPONAME} == "gentoo" ] || exit 0		 # exit if sync'ed repo is NOT gentoo

printf "\n${DATE}  Running ${ME} hooks ..." >> ${LOGFILE}
#********************************************************************************
function getEbuildName() { PKG=`echo "$L" | cut -d: -f2` ; }

function doLog() { echo "${DATE} qrtools: $ME: error: $1" >> ${LOGFILE} ; }

function Result() {
  C=`tput cols` ; C=$(($C-4))
  printf "\033[${C}G"
  case "$EC" in
	"0") printf "\033[32m%s\033[0m\n" "OK";;
	"X") printf "\033[33m%s\033[0m\n" "--" ;;
	  *) printf "\033[1m\033[91m%s\033[0m\n" "!!";;
  esac
}
#
function doRename() {
  getEbuildName
  echo -n "Renaming package: $PKG "
  if [ -d "${REPOPATH}/${PKG}" ] ; then
	mv "${REPOPATH}/${PKG}" "${REPOPATH}/${PKG}-qrt" ; EC=$?
   else
    EC="X"
  fi
  Result
  [ "$EC" != "0" ] && doLog "couldn't rename ${PKG}"

}
#
function doDelete() {
  getEbuildName
  echo -n "Deleting package: $PKG"
  if [ -d "${REPOPATH}/${PKG}" ] ; then
    rm -rf "${REPOPATH}/${PKG}" "${REPOPATH}/${PKG}-qrt" ; EC=$?
   else
    EC="X"
  fi
  Result
  [ "$EC" != "0" ] && doLog "couldn't delete ${PKG}"
}
#********************************************************************************
# development only!
# this function will be replaced by using /etc/portage/env/<category>/<package>
function doPatch() {
  getEbuildName
  local EBUILD
#echo 0: $PKG
  # first we check if a patch exists overall
  FLDR=`echo "$PKG" | cut -d\/ -f2`		# the package ebuild folder
  PATCH="/mnt/NAS/Projects/Gentoo R tools/$FLDR.patch"

  [ -f "$PATCH" ] || return #&& echo found || echo "not found"
  echo -n "Patching package: $PKG"

  # get the latest not masked ebuild version and get the correct line
  EBUILD=`emerge -pv --nodeps "$PKG" | grep ^'\[ebuild' | grep "$PKG"` \
	2>&1>/dev/null		# ... suppress any output from grep
  # split the line: two steps to get the ebuild name w/o the path
  EBUILD=`echo "$EBUILD" | cut -d\] -f2 | awk '{ print \$1; }'`
  EBUILD=`echo "$EBUILD" | cut -d\: -f1 | cut -d\/ -f2`
#echo 2: "$EBUILD"
  # now we put all together with the absolute path
  EBUILD="${REPOPATH}/${PKG}/${EBUILD}.ebuild"
#echo 4: EBUILD: $EBUILD

  patch --dry-run "${EBUILD}" < "$PATCH" 2>&1>/dev/null && \
  patch -f "${EBUILD}" < "$PATCH" 2>&1>/dev/null
  EC=$?
  # the patch could fail because it was applied already --> TODO
#  if [ "$EC" != "0" ] ; then
#	[ -f "${EBUILD}.orig" ] && echo " - done already"
#  fi

  EC=$? ; Result


  if [ "$EC" = "0" ] ; then
	( pushd "${REPOPATH}/${PKG}" 
	  repoman manifest
	  popd ) 2>&1>/dev/null
  fi
#  Result #0 || Result 1  #echo failed

}
# end development doPatch()
#********************************************************************************

function ReadConfigFile() {
  while IFS= read L ; do
	ACTION=`echo "$L" | cut -d: -f1`
	case "$ACTION" in
	  D) doDelete;;
	  R) doRename;;
	  P) doPatch;;
	esac
	PKG=""
  done < "$CFGFILE"
  echo
}

if [ ${DBG} -gt 0 ] ; then
   echo "$DATE REPONAME: ${REPONAME}"  >> ${LOGFILE}
   echo "$DATE REPOPATH: ${REPOPATH}"  >> ${LOGFILE}
   echo "$DATE SYNC_URI: ${SYNC_URI}"  >> ${LOGFILE}
fi

#
printf " \033[92m*\033[0m \033[1mExecuting qrtools' postsync hooks ...\033[0m\n"
if [ -f "$CFGFILE" ] ; then
   ReadConfigFile
 else
   printf " \033[91m>>>\033[0m NO CONFIG FILE FOUND! ABORTING!"
   EC="1" ; Result
   echo                                 # print an empty line
   doLog "file ${CFGFILE} not found"    # additional write to logfile
   exit 0
fi
