#********************************************************************************
# a wrapper around genkernel                                                    *
#                                                                               *
#  Author: Kai Peter, ©2016-2019                                                *
# Version: 0.29                                                                 *
# License: ISC                                                                  *
#                                                                               *
# Description: invokes and runs genkernel with mostly used option               *
#********************************************************************************

RS="\033[91m*\033[0m"      # red star
GS="\033[92m*\033[0m"      # green star

if [ $1 ] ; then
  [ $1 == "view" ] && FLAG="view"
  [ $1 == "save" ] && FLAG="save"
 else
  FLAG=0
fi

# determine the kernel version of the sources and the running kernel
# 'NK' is the 'N'ew 'K'ernel version which should be build
# 'RK' is the actual 'R'unning 'K'ernel version
NK=$(echo `readlink /usr/src/linux` | cut -d- -f2)  # link of /usr/src/linux
RK=$(echo `uname -r` | cut -d- -f1)             # the running kernel version

#NK="3.4.15"  #TEST!
declare -i N=$(echo "$NK" | cut -d. -f1-3 --output-delimiter='' | tr -d '\0')
declare -i R=$(echo "$RK" | cut -d. -f1-3 --output-delimiter="" | tr -d '\0')
# downgrade isn't allowed, we require an integer comparison ...
if [ $N -lt $R ] ; then
  printf "$RS\033[1m qkernel: >> Building an older kernel version than "
  printf "the actual running version isn't supported! Aborting!\033[0m\n"
  exit 1
fi


if [ ! -f "/usr/src/linux/.qconfig" ] ; then
   # check for and use 'proc/config.gz'
   if [ ! -f "/proc/config.gz" ] ; then
      printf "$RS\033[1m qkernel: >> required file '/proc/config.gz' not found! "
      printf "Aborting!\033[0m\n"
      exit 1
   fi
   echo -e "\033[92m*\033[0m qkernel: >> Using /proc/config.gz ..."
   zcat /proc/config.gz > /usr/src/linux/.qconfig
fi
# prevent genkernel message 'files differs!'
cp /usr/src/linux/.qconfig /usr/src/linux/.config

# define option depending if we want to build a newer kernel or not
if [ $N -eq $R ] ; then
   OPT="--kernel-config=/usr/src/linux/.qconfig"
   echo -e "$GS qkernel: >> rebuild the actual running kernel"
  else
   OPT="--oldconfig"
   echo -e "$GS qkernel: >> build a newer kernel version"
fi

# set standard optiions independant of '/etc/genkernel.conf'
OPTS="$OPT --menuconfig --no-save-config"
# build only if no option was given
if [ "$FLAG" -eq 0 ] ; then
  genkernel ${OPTS} all || exit 1
  FLAG="save"
 else
  cd /usr/src/linux && make menuconfig 2>/dev/null
fi

# save the last configuration in '.qconfig'
[ "$FLAG" = "save" ] && cp /usr/src/linux/.config /usr/src/linux/.qconfig
